/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.user.roles;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import org.onosproject.nemo.model.common.UserRoleDescription;
import org.onosproject.nemo.model.common.UserRoleName;

/**
 * The builder of user role.
 */
public class UserRoleBuilder {

    private Map<UserRole, UserRole> augmentation = new HashMap<>();
    private UserRoleKey key;
    private UserRoleDescription roleDescription;
    private UserRoleName roleName;

    /**
     * Creates a default user role builder.
     */
    public UserRoleBuilder() {
    }

    /**
     * Creates a user role builder from a user role object.
     *
     * @param base the user role object
     */
    public UserRoleBuilder(UserRole base) {
        if (base.getKey() == null) {
            key = new UserRoleKey(base.getRoleName());
            roleName = base.getRoleName();
        } else {
            key = base.getKey();
            roleName = key.getRoleName();
        }
        roleDescription = base.getRoleDescription();
    }

    /**
     * Sets the key to be used by the builder.
     *
     * @param key key value
     * @return self
     */
    public UserRoleBuilder key(UserRoleKey key) {
        this.key = key;
        return this;
    }

    /**
     * Sets the description to be used by the builder.
     *
     * @param desc the role description
     * @return self
     */
    public UserRoleBuilder roleDescription(UserRoleDescription desc) {
        roleDescription = desc;
        return this;
    }

    /**
     * Sets the role name to be used by the builder.
     *
     * @param name user role name
     * @return self
     */
    public UserRoleBuilder roleName(UserRoleName name) {
        roleName = name;
        return this;
    }

    /**
     * Builds a user role from this builder instance.
     *
     * @return user role
     */
    public UserRole build() {
        return new UserRoleImpl(this);
    }

    /**
     * The implementation of user role interface.
     */
    private static final class UserRoleImpl implements UserRole {

        private final UserRoleKey key;
        private final UserRoleDescription roleDescription;
        private final UserRoleName roleName;
        private final Map<UserRole, UserRole> augmentation;

        private UserRoleImpl(UserRoleBuilder base) {
            if (base.key == null) {
                key = new UserRoleKey(base.roleName);
                roleName = base.roleName;
            } else {
                key = base.key;
                roleName = key.getRoleName();
            }
            roleDescription = base.roleDescription;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public UserRoleKey getKey() {
            return key;
        }

        @Override
        public UserRoleDescription getRoleDescription() {
            return roleDescription;
        }

        @Override
        public UserRoleName getRoleName() {
            return roleName;
        }

        @Override
        public int hashCode() {
            return Objects.hash(key, roleDescription, roleName, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            UserRole other = (UserRole) obj;
            if (!Objects.equals(roleName, other.getRoleName())) {
                return false;
            }
            if (!Objects.equals(roleDescription,
                    other.getRoleDescription())) {
                return false;
            }
            if (!Objects.equals(key, other.getKey())) {
                return false;
            }
            UserRoleImpl otherImpl = (UserRoleImpl) obj;
            return Objects.equals(augmentation, otherImpl.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("key", key)
                    .add("role description", roleDescription)
                    .add("role name", roleName)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}